from __future__ import absolute_import
from djenga.test.integration_test import IntegrationTestException
from djenga.test.integration_test import IntegrationTest

__all__ = [
    'IntegrationTest',
    'IntegrationTestException',
]



__all__ = [
    'ADJECTIVES',
    'ANIMALS',
]

ADJECTIVES = [
    'adorable',
    'acrobatic',
    'adaptable',
    'adventurous',
    'agile',
    'alert',
    'amiable',
    'amused',
    'arboreal',
    'ardent',
    'artful',
    'astonishing',
    'astute',
    'attentive',
    'authentic',
    'avid',
    'beardless',
    'benevolent',
    'bicameral',
    'big',
    'bionic',
    'blissful',
    'bodacious',
    'brave',
    'bright',
    'brilliant',
    'bubbly',
    'careful',
    'cautious',
    'cavernous',
    'charming',
    'cheeky',
    'cheerful',
    'circumspect',
    'cognizant',
    'cool',
    'collaborative',
    'collectible',
    'colorful',
    'colossal',
    'comfortable',
    'communicative',
    'compact',
    'compassionate',
    'constant',
    'contemplative',
    'contiguous',
    'convincing',
    'convivial',
    'cooperative',
    'courageous',
    'cordial',
    'cosmic',
    'creative',
    'cryptic',
    'crystalline',
    'cunning',
    'curious',
    'dancing',
    'daring',
    'dauntless',
    'dashing',
    'dazzling',
    'defiant',
    'delightful',
    'determined',
    'dexterous',
    'diminutive',
    'discerning',
    'distinctive',
    'dreaming',
    'dynamic',
    'eager',
    'earnest',
    'easygoing',
    'ebullient',
    'eccentric',
    'ecstatic',
    'effluvial',
    'effervescent',
    'elaborate',
    'elated',
    'elegant',
    'eloquent',
    'elusive',
    'enchanting',
    'encouraging',
    'energetic',
    'energized',
    'enormous',
    'enthusiastic',
    'erudite',
    'essential',
    'ethereal',
    'excited',
    'exhilarated',
    'extraordinary',
    'exotic',
    'exuberant',
    'fantastic',
    'fearless',
    'feisty',
    'fierce',
    'fiery',
    'flourishing',
    'flying',
    'focused',
    'fortunate',
    'friendly',
    'frolicking',
    'gargantuan',
    'gesticulating',
    'gigantic',
    'glamorous',
    'gleaming',
    'gleeful',
    'glorious',
    'gorgeous',
    'graceful',
    'grateful',
    'gregarious',
    'happy',
    'harmonious',
    'hatless',
    'healthy',
    'helpful',
    'heroic',
    'hydraulic',
    'hyena',
    'idealistic',
    'illustrious',
    'illustrative',
    'imaginative',
    'immense',
    'impartial',
    'imperturbable',
    'improbable',
    'incredible',
    'inimitable',
    'influential',
    'inquisitive',
    'insightful',
    'inspired',
    'intrepid',
    'intricate',
    'intuitive',
    'invaluable',
    'inventive',
    'iterative',
    'jaunty',
    'jolly',
    'joyful',
    'joyous',
    'jubilant',
    'jumping',
    'katabatic',
    'keen',
    'kempt',
    'kept',
    'kind',
    'kindled',
    'kinematic',
    'kinesthtic',
    'kingly',
    'kitschy',
    'knightly',
    'knowing',
    'kooky',
    'kyphotic',
    'large',
    'laughing',
    'legendary',
    'lenient',
    'lively',
    'loquacious',
    'lucky',
    'luminescent',
    'magnetic',
    'magnificent',
    'majestic',
    'marvelous',
    'masked',
    'massive',
    'mechanical',
    'mercurial',
    'meritorious',
    'merry',
    'methodical',
    'meticulous',
    'mighty',
    'miniature',
    'mirthful',
    'mischievous',
    'modest',
    'momentous',
    'motionless',
    'multicolored',
    'murmuring',
    'musical',
    'mustachioed',
    'mysterious',
    'nascent',
    'neighborly',
    'noble',
    'nomadic',
    'nonchalant',
    'noncommittal',
    'observant',
    'omnidirectional',
    'omnipotent',
    'omnipresent',
    'omnivorous',
    'optimal',
    'optimistic',
    'otherworldly',
    'outgoing',
    'outspoken',
    'panoramic',
    'peaceful',
    'perceptive',
    'perpetual',
    'perplexing',
    'perspicacious',
    'philosophical',
    'picturesque',
    'playful',
    'pleasant',
    'poised',
    'practical',
    'precious',
    'precise',
    'precocious',
    'prestigious',
    'primeval',
    'primordial',
    'prismatic',
    'proactive',
    'proficient',
    'prototypical',
    'prudent',
    'purposeful',
    'qualified',
    'quizzical',
    'quotable',
    'radiant',
    'reassured',
    'reclusive',
    'recursive',
    'recurring',
    'reflective',
    'rejoicing',
    'relaxed',
    'relieved',
    'remarkable',
    'renowned',
    'resilient',
    'resolute',
    'resourceful',
    'rigorous',
    'roaring',
    'robust',
    'salient',
    'salubrious',
    'sanguine',
    'sapient',
    'sarcastic',
    'satisfied',
    'scholarly',
    'scintillating',
    'scrupulous',
    'selective',
    'shiny',
    'sincere',
    'singing',
    'sleek',
    'sleepy',
    'slippery',
    'smiling',
    'smooth',
    'solid',
    'sophisticated',
    'sparkling',
    'spectacular',
    'splendid',
    'spotless',
    'squeaky',
    'stately',
    'steady',
    'strategic',
    'striped',
    'stunning',
    'stupendous',
    'stylish',
    'substantial',
    'sufficient',
    'swimming',
    'symbolic',
    'symmetrical',
    'taciturn',
    'terrestrial',
    'tessellated',
    'thankful',
    'theoretical',
    'thoughtful',
    'thriving',
    'timeless',
    'timely',
    'topographical',
    'transparent',
    'tranquil',
    'transparent',
    'tricky',
    'ubiquitous',
    'uncanny',
    'unclouded',
    'undisputed',
    'unexpected',
    'unfathomable',
    'unflappable',
    'unique',
    'universal',
    'unofficial',
    'unseen',
    'unthinkable',
    'uproarious',
    'variegated',
    'versatile',
    'victorious',
    'vigilant',
    'vigorous',
    'virtuous',
    'vivacious',
    'vivid',
    'vociferous',
    'wacky',
    'wandering',
    'watchful',
    'whimsical',
    'windswept',
    'witty',
    'wonderful',
    'wondrous',
    'yakkety',
    'yawning',
    'yeatsian',
    'yelling',
    'yellow',
    'yellowing',
    'young',
    'youthful',
    'yummy',
    'zany',
    'zaftig',
    'zealous',
    'zestful',
    'zesty',
    'zippy',
    'zoonatic',
    'zygomorphous',
    'zymoid',
    'zymolytic',
]
ANIMALS = [
    'aardvark',
    'albatross',
    'alligator',
    'alpaca',
    'angelfish',
    'anteater',
    'antelope',
    'ape',
    'armadillo',
    'badger',
    'barracuda',
    'bat',
    'beagle',
    'bear',
    'beaver',
    'bird',
    'bittern',
    'bobolink',
    'brontosaurus',
    'boa-constrictor',
    'buffalo',
    'bulldog',
    'bumblebee',
    'butterfly',
    'buzzard',
    'camel',
    'caribou',
    'cassowary',
    'cat',
    'catfish',
    'caterpillar',
    'cauldron',
    'centipede',
    'cete',
    'chameleon',
    'cheetah',
    'chinchilla',
    'chipmunk',
    'cobra',
    'coelacanth',
    'condor',
    'coot',
    'coral-snake',
    'cormorant',
    'crab',
    'crane',
    'crocodile',
    'dalmatian',
    'deer',
    'dolphin',
    'donkey',
    'dotterel',
    'dove',
    'dragonfish',
    'dragonfly',
    'duck',
    'eagle',
    'eel',
    'elephant',
    'elk',
    'falcon',
    'ferret',
    'finch',
    'firefly',
    'fish',
    'flamingo',
    'fox',
    'frog',
    'gang',
    'gazelle',
    'gecko',
    'gerbil',
    'giraffe',
    'gnu',
    'goldfish',
    'goose',
    'gorilla',
    'grasshopper',
    'greyhound',
    'grouse',
    'gull',
    'hamster',
    'hare',
    'hawk',
    'hatchetfish',
    'hedgehog',
    'heron',
    'herring',
    'hornet',
    'horse',
    'hummingbird',
    'ibex',
    'ibis',
    'iguana',
    'jackal',
    'jaguar',
    'jay',
    'jellyfish',
    'kangaroo',
    'kestrel',
    'kingfisher',
    'koala',
    'koi',
    'lapwing',
    'lark',
    'lemur',
    'leopard',
    'lion',
    'lionfish',
    'llama',
    'lobster',
    'loris',
    'magpie',
    'mallard',
    'mandrill',
    'manta-ray',
    'mantis',
    'marlin',
    'mastiff',
    'mollusk',
    'mongoose',
    'moose',
    'mouse',
    'mule',
    'narwhal',
    'nautilus',
    'newt',
    'nightingale',
    'obstinacy',
    'octopus',
    'okapi',
    'opossum',
    'orca',
    'osprey',
    'ostrich',
    'otter',
    'owl',
    'panda',
    'panther',
    'parrot',
    'partridge',
    'pelican',
    'penguin',
    'pheasant',
    'pig',
    'pigeon',
    'platypus',
    'polar-bear',
    'porcupine',
    'porpoise',
    'python',
    'quail',
    'rabbit',
    'raccoon',
    'ram',
    'raven',
    'reindeer',
    'rhinoceros',
    'roadrunner',
    'rook',
    'salamander',
    'salmon',
    'sandpiper',
    'scorpion',
    'sea-cucumber',
    'sea-lion',
    'sea-snake',
    'sea-turtle',
    'seahorse',
    'seal',
    'shark',
    'sheep',
    'shrewdness',
    'sloth',
    'sleuth',
    'snowy-owl',
    'songbird',
    'sparrow',
    'spider',
    'squid',
    'squirrel',
    'starfish',
    'starling',
    'stegosaurus',
    'stingray',
    'stork',
    'swan',
    'tapir',
    'tiger',
    'toucan',
    'triceratops',
    'turtle',
    'vampire-bat',
    'velociraptor',
    'wallaby',
    'water-buffalo',
    'walrus',
    'whale',
    'wolf',
    'wolverine',
    'wombat',
    'wrasse',
    'wren',
    'xantus',
    'xenarthra',
    'xenops',
    'xerus',
    'yabby',
    'yak',
    'yellow-eyed-penguin',
    'yellow-ground-squirrel',
    'yellow-mongoose',
    'yuma-myotis',
    'zebra',
    'zebra-finch',
    'zebrafish',
    'zebra-shark',
    'zebroid',
    'zebu',
    'zonkey',
    'zorilla',
    'zorro',
    'zorse',
    'clowder',      # cats
    'pounce',       # cats
    'kindle',       # kittens
    'litter',       # kittens, dogs
    'intrigue',     # kittens
    'cowardice',    # dogs
    'pace',         # donkeys
    'parade',       # elephants
    'business',     # ferrets
    'shrewdness',   # apes
    'gang',         # buffalo
    'obstinacy',    # buffalo
    'leash',        # foxes
    'skulk',        # foxes
    'earth',        # foxes
    'tower',        # giraffes
    'tribe',        # goats
    'trip',         # goats
    'band',         # gorillas
    'bloat',        # hippos
    'thunder',      # hippos
    'cackle',       # hyenas
    'shadow',       # jaguar
    'troop',        # kangaroo
    'mob',          # kangaroo
    'conspiracy',   # lemurs
    'leap',         # leopards
    'pride',        # lions
    'sawt',         # lions
    'richness',     # martens
    'labor',        # moles
    'troop',        # monkeys
    'barrel',       # monkeys
    'pack',         # mules
    'span',         # mules
    'barren',       # mules
    'romp',         # otter
    'drift',        # pigs
    'drove',        # drove
    'sounder',      # pigs
    'team',         # pigs
    'passel',       # pigs
    'prickle',      # porcupines
    'pod',          # porpoises, whales
    'school',       # porpoises, whales
    'herd',         # porpoises
    'turmoil',      # porpoises
    'colony',       # rabbits
    'warren',       # rabbits
    'nest',         # rabbits
    'down',         # rabbits
    'husk',         # rabbits
    'gam',          # whales
    'route',        # wolves
    'woute',        # wolves
    'sedge',        # bitterns
    'wake',         # buzzard
    'chain',        # bobolink
    'cover',        # coot
    'gulp',         # cormorants
    'murder',       # crows, magpies
    'horde',        # crows
    'trip',         # dotterel
    'dule',         # doves
    'pitying',      # doves
    'brace',        # ducks
    'team',         # ducks
    'flock',        # ducks
    'raft',         # ducks
    'paddling',     # ducks
    'badling',      # ducks
    'convocation',  # eagles
    'charm',        # finches, magpies
    'flamingos',    # stand
    'gaggle',       # geese
    'skein',        # geese
    'cast',         # hawk
    'kettle',       # hawk
    'boil',         # hawk
    'sedge',        # heron
    'siege',        # heron
    'party',        # jays
    'scold',        # jays
    'deceit',       # lapwings
    'exaltation',   # larks
    'sord',         # mallards
    'tiding',       # magpies
    'gulp',         # magpies
    'watch',        # nightingales
    'parliament',   # owls
    'pandemonium',  # parrots
    'company',      # parrots
    'covey',        # partridge
    'ostentation',  # peacocks
    'muster',       # peacocks, penguins
    'parcel',       # penguins
    'rookery',      # rookery
]





#This file uses the following encoding: utf-8


import logging


__all__ = [
    u'VerbosityLevels',
    u'is_ducktyped_logger',
]

class VerbosityLevels(object):
    SUMMARY = 0
    ERROR = 1
    WARNING = 2
    INFO = 3

    VERBOSITY_TO_LOGGING = {
        SUMMARY: logging.CRITICAL,
        ERROR: logging.ERROR,
        WARNING: logging.WARNING,
        INFO: logging.DEBUG,
    }

    @staticmethod
    def to_logging_level(verbosity):
        return VerbosityLevels.VERBOSITY_TO_LOGGING.get(verbosity, logging.DEBUG)


def is_ducktyped_logger(log):
    return (hasattr(log, u'critical') and
            hasattr(log, u'error') and
            hasattr(log, u'warning') and
            hasattr(log, u'info') and
            hasattr(log, u'debug'))








import codecs
import csv
# from django.core.management.base import BaseCommand


class InputCsvMixin(object):
    """
    Enhances a django management command by allowing it to
    read from an input CSV file through an
    instance function called `read_input`.  The input file
    may be specified at the command line by using the
    `--input` or `-i` option.

    n.b. This mixin requires that any sub-classes making use of
    it to call super(ClassName, self).add_arguments(parser)
    at the beginning of any override of the `add_arguments`
    method.
    """
    def __init__(self):
        super(InputCsvMixin, self).__init__()
        self.input_rows = []
        """@type: list[list]"""
        self.header = []
        """@type: list[str]"""
        self.input_file_help = 'the /path/to the input CSV file'
        """@type: str"""
        self.input_filename = None
        """@type: str"""

    def add_arguments(self, parser):
        """
        Adds the input file argument to the option
        parser
        :param parser: argparse.Parser
        """
        super(InputCsvMixin, self).add_arguments(parser)
        parser.add_argument(
            '-i', '--input',
            dest='input_file',
            required=True,
            help=self.input_file_help,
        )

    def read_input(self):
        """
        Reads the input file specified by the user into the
        `self.header` and `self.input_rows` instance variables.
        :return:
        """
        with codecs.open(self.input_filename, 'r', encoding='utf-8') as f:
            reader = csv.reader(f)
            self.header = reader.next()
            self.input_rows = [ x for x in reader ]

    def read_input_as_dict(self):
        """
        Reads the input file specified by the user
        into the `self.input_rows` instance variable
        as a series of dicts using csv.DictWriter
        :return:
        """
        with codecs.open(self.input_filename, 'r', encoding='utf-8') as f:
            reader = csv.DictReader(f)
            self.input_rows = [ x for x in reader ]

    def execute(self, *args, **options):
        """
        Overrides the `BaseCommand.execute` method to allow
        us to grab the input filename argument from
        the parser and store it into `self.input_filename`
        :param args:
        :param options:
        :return:
        """
        self.input_filename = options['input_file']
        return super(InputCsvMixin, self).execute(*args, **options)




from .dictionary import ADJECTIVES, ANIMALS
import random


__all__ = [ 'animal_pair' ]


def animal_pair():
    return u'%s-%s' % (
        random.choice(ADJECTIVES),
        random.choice(ANIMALS),
    )




def chunkify(rg, n_max=1000):
    """
    generates sublists of at most n_max
    elements from the provided array
    :param rg: list
    :param n_max: the maximum sublist length
    :return:
    """
    m, n_len = 0, len(rg)
    while m < n_len:
        l, m = m, m + n_max
        yield rg[l:m]




import os
from io import BytesIO
from zipfile import ZipFile, ZIP_DEFLATED
from StringIO import StringIO
from djenga.csv import UnicodeCsvWriter
from wsgiref.util import FileWrapper
from django.http import HttpResponse
from django.http import StreamingHttpResponse
from django.views.generic import View


class CsvViewMixin(object):
    """
    This class offers several utility functions to
    make it easier to return a CSV response to a user.
    To use:
        1.  Add this class as a mixin to any class-based view
        2.  Call self.initialize_response with a proposed download filename
        3.  write data to the CSV response with self.writerow
        4.  return self.response from the get or post function to send the
            CSV to the user
    """
    def __init__(self, **kwargs):
        super(CsvViewMixin, self).__init__(**kwargs)
        self.response = None
        """:type HttpResponse"""
        self.writer = None
        """:type csv.writer"""

    def initialize_response(self, filename):
        """
        Prepares the HttpResponse that will be used
        to contain the CSV data
        :param filename: the default filename for the user
        when he/she downloads the file.
        """
        self.response = HttpResponse(content_type='text/csv')
        self.response['Content-Disposition'] = 'attachment; filename="%s"' % filename
        self.writer = UnicodeCsvWriter(self.response)

    def writerow(self, data):
        self.writer.writerow(data)


class CsvView(CsvViewMixin, View):
    """
    A helper class so that you can just subclass from
    CsvView directly instead of using the mixin.
    """
    pass


class ZippedCsvMixin(object):
    """
    This mixin offers several utility functions to
    make it easier to return a CSV response to a user.
    To use:
        1.  Add this class as a mixin
        2.  Call self.initialize_response with a proposed download filename
        3.  write data to the CSV response with self.writerow
        4.  call self.finalize_response to finalize the response
        4.  return self.response from the get or post function to send the
            CSV to the user
    """
    def __init__(self, **kwargs):
        super(ZippedCsvMixin, self).__init__(**kwargs)
        self.response = None
        """:type HttpResponse"""
        self.writer = None
        """:type csv.writer"""
        self.csv_buffer = StringIO()
        self.zip_buffer = BytesIO()
        self.filename = None
        self.archive = None
        self.response = None

    def initialize_response(self, filename):
        """
        Prepares the HttpResponse that will be used
        to contain the CSV data
        :param filename: the default filename for the user
        when he/she downloads the file.
        """
        self.writer = UnicodeCsvWriter(self.csv_buffer)
        self.filename = filename
        self.archive = ZipFile(self.zip_buffer, 'w', compression=ZIP_DEFLATED)

    def finalize_response(self):
        self.archive.writestr(self.filename, self.csv_buffer.getvalue().encode('utf-8'))
        self.archive.close()
        _, ext = os.path.splitext(self.filename)
        self.response = StreamingHttpResponse(FileWrapper(self.zip_buffer), content_type='application/zip')
        self.response['Content-Disposition'] = 'attachment; filename="%s"' % self.filename.replace(ext, '.zip')
        self.response['Content-Length'] = self.zip_buffer.tell()
        self.zip_buffer.seek(0)

    def writerow(self, data):
        self.writer.writerow(data)


class ZippedCsvView(ZippedCsvMixin, View):
    """
    A helper class so that you can just subclass from
    ZippedCsvView directly instead of using the mixin.
    """
    pass



from __future__ import absolute_import, unicode_literals, print_function
import logging
import logging.handlers
import os


__all__ = [
    'ColorFormatter',
    'BriefColorFormatter',
    'WorldWritableRotatingFileHandler',
    'IgnoreDjangoInternals',
    'IgnorePisaPdf',
    'IgnoreRequests',
    'IgnoreCaching',
]


class _Constants(object):
    LEVEL_COLORS = {
        """
        Maps a logging level to a pair of colors:
            first color is the label color
            second color is the text color
        """
        'DEBUG':    (33,  39),
        'TRACE':    (147, 153),
        'INFO':     (43,  49),
        'WARNING':  (214, 226),
        'ERROR':    (196, 197),
        'CRITICAL': (196, 197),
    }

    SHORT_LEVELS = {
        u'INFO': u'INF',
        u'WARNING': u'WRN',
        u'ERROR': u'ERR',
        u'DEBUG': u'DBG',
        u'EXCEPTION': u'EXC',
        u'CRITICAL': u'CRT'
    }

    NOT_EXTRA = {
        'args',
        'created',
        'exc_info',
        'exc_text',
        'filename',
        'funcName',
        'getMessage',
        'levelname',
        'levelno',
        'lineno',
        'message',
        'module',
        'msecs',
        'msg',
        'name',
        'pathname',
        'process',
        'processName',
        'relativeCreated',
        'thread',
        'threadName',
    }


class ColorFormatter(logging.Formatter):
    def format(self, record):
        level_colors = _Constants.LEVEL_COLORS.get(record.levelname, (33, 39))
        # setup the 256-color spectrum
        st_color   = "\033[38;5;{:d}m"
        st_reset   = "\033[0m"
        message = logging.Formatter.format(self, record)

        # Pass any simple messages from internal things,
        # like Django's runserver, without special formatting.
        if record.name == 'werkzeug' and record.levelname == 'INFO':
            # Highlight POST verbs.
            if '] "POST ' in record.message:
                record.message = record.message.replace(
                    '] "POST ',
                    '] "\033[38;5;85mPOST\033[0m ')
            return record.message
        # no highlighting for DEBUG messages
        if record.name.startswith('django.') and record.levelname == 'DEBUG':
            return record.message

        # Log entries can have data sent in the `extra` parameter;
        # e.g., when calling logger.error('msg', extra={''})
        #  Format extra to include its values in the log print.
        mp_extra = {}
        n_max = 0
        st_extra = ''
        for key in set(dir(record)) - _Constants.NOT_EXTRA:
            if key.startswith('_') or key == 'stack_info':
                continue
            mp_extra[key] = getattr(record, key)
            if len(key) > n_max:
                n_max = len(key)
        if mp_extra:
            st_extra = u'\n%s' % (u'\n'.join([
                '    {key:>{n_max}}: {value!r}'.format(
                    key=key,
                    value=value,
                    n_max=n_max
                ) for key, value in mp_extra.items()
            ]),)

        return '%s[%s/%s]%s %s%s%s' % (
            st_color.format(level_colors[0]),
            record.levelname,
            record.name,
            st_color.format(level_colors[1]),
            message,
            st_extra,
            st_reset,
        )


class BriefColorFormatter(logging.Formatter):
    def format(self, record):
        level_colors = _Constants.LEVEL_COLORS.get(record.levelname, (33, 39))
        st_color   = "\033[38;5;{:d}m"
        st_reset   = "\033[0m"
        message = logging.Formatter.format(self, record)

        # Pass any simple messages from internal things,
        # like Django's runserver, without special formatting.
        if record.name == 'werkzeug' and record.levelname == 'INFO':
            # Highlight POST verbs.
            if '] "POST ' in record.message:
                record.message = record.message.replace(
                    '] "POST ',
                    '] "\033[38;5;85mPOST\033[0m ')
            return record.message
        # no highlighting for DEBUG messages
        if record.name.startswith('django.') and record.levelname == 'DEBUG':
            return record.message

        # Log entries can have data sent in the `extra` parameter;
        # e.g., when calling logger.error('msg', extra={''})
        #  Format extra to include its values in the log print.
        mp_extra = {}
        n_max = 0
        st_extra = ''
        for key in set(dir(record)) - _Constants.NOT_EXTRA:
            if key.startswith('_') or key == 'stack_info':
                continue
            mp_extra[key] = getattr(record, key)
            if len(key) > n_max:
                n_max = len(key)
        if mp_extra:
            st_extra = u'\n%s' % (u'\n'.join([
                '    {key:>{n_max}}: {value!r}'.format(
                    key=key,
                    value=value,
                    n_max=n_max
                ) for key, value in mp_extra.items()]),)
        return u'%s[%s]%s %s%s%s' % (
            st_color.format(level_colors[0]),
            _Constants.SHORT_LEVELS.get(record.levelname, 'GEN'),
            st_color.format(level_colors[1]),
            message,
            st_extra,
            st_reset,
        )


class WorldWritableRotatingFileHandler(logging.handlers.TimedRotatingFileHandler):
    def _open(self):
        """Open a new log file with a more permissive umask."""
        # This is due to jobs-12 running manage.py cron jobs as a different user than what Celery runs as;
        #   otherwise we'll run into permission errors because the default RotatingFileHandler opens them
        #   as user writable, not world writable.
        old_umask = os.umask(0o111) # 666
        new_file  = logging.handlers.TimedRotatingFileHandler._open(self)
        os.umask(old_umask)
        return new_file


class IgnoreDjangoInternals(logging.Filter):
    def filter(self, record):
        if record.name.startswith('django') and record.levelname == 'DEBUG':
            return False
        return True


class IgnorePisaPdf(logging.Filter):
    def filter(self, record):
        return not (
            record.name.startswith('xhtml2pdf') and
            record.levelname == 'DEBUG'
        )


class IgnoreRequests(logging.Filter):
    def filter(self, record):
        return not (
            record.name.startswith('requests') and
            record.levelname in ('INFO', 'DEBUG')
        )


class IgnoreCaching(logging.Filter):
    def filter(self, record):
        return not (
            record.name.startswith('caching') and
            record.levelname == 'DEBUG'
        )




# encoding: utf-8


from __future__ import unicode_literals
import logging
from datetime import date


logger = logging.getLogger(__name__)
__all__ = [
    u'unpack_date',
    u'bump',
    u'bump_down',
]


def unpack_date(d=None):
    """
    _unpack_date is a quick utility function that
    takes a date and unpacks it to its day, month and
    year values.

    @param d the date to be unpacked.  If no date is
    passed in, then today's date is unpacked.

    @return a tuple with day, month and year (in that order)
    """
    if not d:
        d = date.today()
    return d.day, d.month, d.year


def bump(month, year):
    """
    bump is a utility function that takes a given month
    and year and returns the next month and year.  For
    example bump(1, 2014) would return (2, 2014) while
    bump(12, 2013) would return (1, 2014).

    @param month the month to be bumped
    @param year the year to be bumped

    @return a tuple with the next month and year in that order
    """
    month = month % 12 + 1
    if month == 1:
        year += 1
    return month, year


def bump_down(month, year):
    """
    bump is a utility function that takes a given month
    and year and returns the next month and year.  For
    example bump(1, 2014) would return (2, 2014) while
    bump(12, 2013) would return (1, 2014).

    @param month the month to be bumped
    @param year the year to be bumped

    @return a tuple with the next month and year in that order
    """
    month -= 1
    if month == 0:
        year -= 1
        month = 12
    return month, year



#encoding: utf-8

from __future__ import unicode_literals
import logging
from base64 import b64decode
from base64 import b64encode
from Crypto.Cipher import Blowfish
from Crypto import Random
from struct import pack
import settings


__all__ = [
    u'encrypt',
    u'decrypt',
]


logger = logging.getLogger(__name__)


def encrypt(s):
    """
    The encrypt function encrypts a unicode string using the
    Blowfish cipher (provided by pycrypto).  The key used is
    the SECRET_KEY specified in the settings file.

    @param s The plaintext unicode string to be encrypted.

    @return The encrypted ciphertext.
    """

    block_size = Blowfish.block_size
    iv = Random.new().read(block_size)
    if not isinstance(s, basestring):
        s = '%s' % s
    b_plaintext = b64encode(s.encode('utf-8'))
    plen = block_size - (len(b_plaintext) % 8)
    padding = [plen] * plen
    padding = pack('b' * plen, *padding)
    e_cipher = Blowfish.new(settings.DJENGA_ENCRYPTION_KEY, Blowfish.MODE_CBC, iv)
    ciphertext = iv + e_cipher.encrypt(b_plaintext + padding)
    return ciphertext


def decrypt(s):
    """
    The decrypt function decrypts unicode strings that have
    been encrypted by the util.encryption.encrypt() function.
    The cipher used is Blowfish (provided by pcrypto), and the
    key used is the SECRET_KEY specified in the settings file.

    @param s The encrypted ciphertext.

    @return The decrypted plaintext (unicode) string.
    """

    block_size = Blowfish.block_size
    d_cipher = Blowfish.new(settings.DJENGA_ENCRYPTION_KEY, Blowfish.MODE_CBC)
    d_b64text = d_cipher.decrypt(s)
    plaintext = b64decode(d_b64text[block_size:]).decode('utf-8')
    return plaintext




# encoding: utf-8
# pylint: disable=pointless-string-statement

from __future__ import unicode_literals
import threading
from time import time
from django.db import connections


timers = threading.local()


def query_count():
    n_queries = 0
    for x in connections:
        n_queries += len(connections[x].queries)
    return n_queries


def start_timer(logger, name, count_queries=False):
    """
    @type logger: logging.Logger | djenga.mixins.LoggingMixin
    @type name: basestring
    @type count_queries: bool
    """
    if not hasattr(timers, 'stack'):
        timers.stack = list()
        """@type: list[(logging.Logger, basestring, float, bool, int)]"""
    stack = timers.stack
    tm_start = time()
    n_queries = query_count()
    stack.append((logger, name, tm_start, count_queries, n_queries))
    logger.info('%sStarting %s', '-' * len(stack), name)


def end_timer():
    if not hasattr(timers, 'stack'):
        timers.stack = list()
        """@type: list[(logging.Logger, basestring, float, bool, int)]"""
    stack = timers.stack
    logger, name, tm_start, count_queries, queries = stack.pop()
    tm_end = time()
    logger.info(
        '%s%s took %.1fms',
        '-' * (len(stack) + 1),
        name,
        1000 * (tm_end-tm_start)
    )
    if count_queries:
        queries_end = query_count()
        logger.info('Query count: %d', queries_end-queries)



# encoding: utf-8
from __future__ import absolute_import
from djenga.loggers.util import ColorFormatter
from djenga.loggers.util import BriefColorFormatter
from djenga.loggers.util import IgnoreCaching
from djenga.loggers.util import IgnoreDjangoInternals
from djenga.loggers.util import IgnorePisaPdf
from djenga.loggers.util import IgnoreRequests
from djenga.loggers.util import WorldWritableRotatingFileHandler

__all__ = [
    'ColorFormatter',
    'BriefColorFormatter',
    'IgnoreCaching',
    'IgnoreDjangoInternals',
    'IgnorePisaPdf',
    'IgnoreRequests',
    'WorldWritableRotatingFileHandler',
]



"""
This module provides functions for rounding numbers as currency.

>>> currency_round_down(1.9999)
Decimal('1.99')
>>> currency_round_up(1.9911)
Decimal('2.00')
>>> currency_round_up4(1.9911)
Decimal('1.9912')
>>> currency_round_half_up(1.9951)
Decimal('2.00')
>>> currency_round_half_up4(1.995555)
Decimal('1.9956')
"""

# encoding: utf-8


from __future__ import unicode_literals
from decimal import Decimal
from decimal import ROUND_FLOOR
from decimal import ROUND_DOWN
from decimal import ROUND_UP
from decimal import ROUND_HALF_UP


__all__ = [
    u'currency_round_down',
    u'currency_round_up',
    u'currency_round_up4',
    u'currency_round_half_up',
    u'currency_round_half_up4',
]


def currency_round_down(amount):
    """
    Useful helper function that takes a numerical amount, converts
    it to a decimal.Decimal object and rounds it down to the nearest
    cent.
    """
    amount = amount or 0.00
    amount = Decimal(amount)
    amount = amount.quantize(Decimal(u'.0001'), rounding=ROUND_FLOOR)
    amount = amount.quantize(Decimal(u'.01'), rounding=ROUND_DOWN)
    return amount


def currency_round_up(amount):
    """
    Useful helper function that takes a numerical amount, converts
    it to a decimal.Decimal object and rounds it up to the nearest
    cent.
    """
    amount = amount or 0.00
    amount = Decimal(amount)
    amount = amount.quantize(Decimal(u'.0001'), rounding=ROUND_FLOOR)
    amount = amount.quantize(Decimal(u'.01'), rounding=ROUND_UP)
    return amount


def currency_round_up4(amount):
    """
    Useful helper function that takes a numerical amount, converts
    it to a decimal.Decimal object and rounds it up to the nearest
    one-hundredth of a cent.
    """
    amount = amount or 0.00
    amount = Decimal(amount)
    amount = amount.quantize(Decimal(u'.0001'), rounding=ROUND_UP)
    return amount


def currency_round_half_up(amount):
    """
    Useful helper function that takes a numerical amount, converts
    it to a decimal.Decimal object and rounds it up to the nearest
    cent using half-adjust rounding.  Unlike `currency_round_up`, this
    function will use half-adjust rounding for the cents place after
    floor rounding the ten thousandths place.
    """
    amount = amount or 0.00
    amount = Decimal(amount)
    amount = amount.quantize(Decimal(u'.0001'), rounding=ROUND_FLOOR)
    amount = amount.quantize(Decimal(u'.01'), rounding=ROUND_HALF_UP)
    return amount


def currency_round_half_up4(amount):
    """
    Useful helper function that takes a numerical amount, converts
    it to a decimal.Decimal object and rounds it up to the nearest
    one-hundredth of a cent using half-adjust rounding.  Unlike
    `currency_round_up4`, this function will use half-adjust rounding
    for the ten thousandths place.
    """
    amount = amount or 0.00
    amount = Decimal(amount)
    amount = amount.quantize(Decimal(u'.0001'), rounding=ROUND_HALF_UP)
    return amount



# encoding: utf-8
import logging


__all__ = [
    'update_progress',
]
logger = logging.getLogger(__name__)


def update_progress(task, progress, *args, **kwargs):
    """
    Adds progress and additional metadata to a celery task.  Since
    celery tasks can also be called as regular ol’ methods, we have
    to handle cases when task.request.id is None (i.e., task.request.id
    is None when a task is called as a regular function instead of with
    `delay` or `apply_async`)
    :param task: celery.Task
    :param progress: format string
    :param args: args for the progress format string
    :param kwargs: additional metadata field/values to set on the task
    """
    if args:
        progress %= args
    request_id = task.request.id
    current = getattr(task.request, 'progress', [])
    current.append(progress)
    if request_id:
        task.backend.mark_as_started(
            request_id,
            progress='\n'.join(current),
            **kwargs)
    setattr(task.request, 'progress', current)
    values = getattr(task.request, 'info', {})
    for key, value in kwargs.items():
        values[key] = value
    setattr(task.request, 'info', values)
    logger.info(progress)



from __future__ import absolute_import
from djenga.profiling.memory import *
from djenga.profiling.timing import start_timer
from djenga.profiling.timing import end_timer

__all__ = [
    'start_timer',
    'end_timer',
    'get_memory_usage',
    'get_peak_memory_usage',
]





import platform
import os


__all__ = [ 'get_memory_usage', 'get_peak_memory_usage' ]


def get_memory_usage():
    try:
        import psutil
        process = psutil.Process(os.getpid())
        return process.memory_info().rss
    except ImportError:
        return None


def get_peak_memory_usage():
    os_name = platform.platform(terse=True).lower()
    if 'windows' in os_name:
        try:
            from wmi import WMI
            w = WMI('.')
            result = w.query("SELECT WorkingSet FROM Win32_PerfRawData_PerfProc_Process WHERE IDProcess=%d" % os.getpid())
            return int(result[0].WorkingSet)
        except ImportError:
            return None
    try:
        import resource
        return resource.getrusage(resource.RUSAGE_SELF).ru_maxrss
    except ImportError:
        return None



# encoding: utf-8

import csv
import six
try:
    from cStringIO import StringIO
except ImportError:
    from io import StringIO


class UnicodeCsvWriter(object):
    def __init__(self, f, dialect=csv.excel, **kwds):
        self.queue = StringIO()
        self.writer = csv.writer(self.queue, dialect=dialect, **kwds)
        self.stream = f

    def writerow(self, row):
        """
        writerow(unicode) -> None
        This function takes a Unicode string and encodes it to the output.
        """
        data = []
        basestring_type = six.string_types[0]
        for value in row:
            if not isinstance(value, basestring_type):
                value = '%s' % (value,)
            data.append(value.encode('utf-8'))
        self.writer.writerow(data)
        data = self.queue.getvalue()
        data = data.decode('utf-8')
        self.stream.write(data)
        self.queue.truncate(0)

    def writerows(self, rows):
        for row in rows:
            self.writerow(row)




from __future__ import absolute_import
from djenga.utils.print_utils import flush_print
from djenga.utils.enum_utils import ReversibleEnum
from djenga.utils.enum_utils import LookupEnum
from djenga.utils.list_utils import chunkify


__all__ = [
    'flush_print',
    'ReversibleEnum',
    'LookupEnum',
    'chunkify',
]



import codecs
import csv
import six


class Utf8Recoder(object):
    """
    Iterator that reads an encoded stream and reencodes the input to UTF-8
    """
    def __init__(self, f):
        super(Utf8Recoder, self).__init__()
        self.reader = f

    def __iter__(self):
        return self

    def next(self):
        return self.reader.next().encode('utf-8')


if six.PY2:
    class UnicodeCsvReader(object):
        """
        A CSV reader which will iterate over lines in the CSV file "f",
        which is encoded in the given encoding.
        """
        def __init__(self, f, dialect=csv.excel, **kwargs):
            super(UnicodeCsvReader, self).__init__()
            f = Utf8Recoder(f)
            self.reader = csv.reader(f, dialect=dialect, **kwargs)

        def next(self):
            row = next(self.reader)
            return [ unicode(s, 'utf-8') for s in row ]

        def __iter__(self):
            return self

    def read_unicode_csv(filename, encoding='utf-8'):
        with codecs.open(filename, 'r', encoding=encoding) as f:
            reader = UnicodeCsvReader(f)
            for x in reader:
                yield x

elif six.PY3:
    UnicodeCsvReader = csv.reader

    def read_unicode_csv(filename, encoding='utf-8'):
        with open(filename, 'r', encoding=encoding) as f:
            reader = UnicodeCsvReader(f)
            for x in reader:
                yield x




from __future__ import print_function
import sys
import re
from functools import partial


def flush_print(st, *args, **kwargs):
    end = kwargs.pop('end', '\n')
    if args:
        st = st % args
    print(st, end=end)
    if sys.stdout.isatty():
        sys.stdout.flush()


def cprint(color_fn, st, *args):
    if args:
        st = st % args
    print(color_fn(st), end='')
    sys.stdout.flush()


def dot_lead(st, *args, **kwargs):
    width = kwargs.pop('width', 60)
    if args:
        st = st % args
    dots = '.' * (width - len(st))
    return '%s%s' % (st, dots)


def dot_leader(st, *args, **kwargs):
    width = kwargs.pop('width', 60)
    if args:
        st = st % args
    dots = '.' * (width - len(st))
    flush_print('%s%s', st, dots, **kwargs)


COLORS = (
    'black', 'red', 'green', 'yellow',
    'blue', 'magenta', 'cyan', 'white')
STYLES = (
    'bold', 'faint', 'italic', 'underline',
    'blink', 'blink2', 'negative',
    'concealed', 'crossed')


def color(s, fg=None, bg=None, style=None):
    sgr = []

    if fg:
        if fg in COLORS:
            sgr.append('%s' % (30 + COLORS.index(fg),))
        elif isinstance(fg, int) and 0 <= fg <= 255:
            sgr.append('38;5;%d' % int(fg))
        else:
            raise Exception('Invalid color [%s]' % fg)

    if bg:
        if bg in COLORS:
            sgr.append(str(40 + COLORS.index(bg)))
        elif isinstance(bg, int) and 0 <= bg <= 255:
            sgr.append('48;5;%d' % bg)
        else:
            raise Exception('Invalid color "%s"' % bg)

    if style:
        for st in style.split('+'):
            if st in STYLES:
                sgr.append(str(1 + STYLES.index(st)))
            else:
                raise Exception('Invalid style "%s"' % st)

    if sgr:
        prefix = '\x1b[' + ';'.join(sgr) + 'm'
        suffix = '\x1b[0m'
        return prefix + s + suffix
    else:
        return s


def strip_color(s):
    return re.sub('\x1b\[.+?m', '', s)


# Foreground shortcuts
black = partial(color, fg='black')
red = partial(color, fg='red')
green = partial(color, fg='green')
yellow = partial(color, fg='yellow')
blue = partial(color, fg='blue')
magenta = partial(color, fg='magenta')
cyan = partial(color, fg='cyan')
white = partial(color, fg='white')

# Style shortcuts
bold = partial(color, style='bold')
faint = partial(color, style='faint')
italic = partial(color, style='italic')
underline = partial(color, style='underline')
blink = partial(color, style='blink')
blink2 = partial(color, style='blink2')
negative = partial(color, style='negative')
concealed = partial(color, style='concealed')
crossed = partial(color, style='crossed')




from djenga.dateutil.basic_functions import unpack_date
from djenga.dateutil.basic_functions import bump
from djenga.dateutil.basic_functions import bump_down

__all__ = [
    'unpack_date',
    'bump',
    'bump_down',
]



# This python file uses the following encoding: utf-8
# pylint: disable=pointless-string-statement


import logging
import os
from django.template import loader
from django.core.mail import EmailMultiAlternatives
from settings import DJENGA_FROM_EMAIL
from settings import DJENGA_DEFAULT_RECIPIENTS
import inlinecss

logger = logging.getLogger(__name__)


__all__ = [
    u'send_email',
    u'send_html_email',
]


def send_email(sender, recipients, subject, body):
    try:
        sender = sender or DJENGA_FROM_EMAIL
        recipients = recipients or DJENGA_DEFAULT_RECIPIENTS
        msg = EmailMultiAlternatives(subject, body, sender, recipients)
        msg.send()
    except BaseException, ex: # pylint:disable=broad-except
        logger.exception(u'Exception encountered trying to send error e-mail:  %s', ex)


def send_html_email(subject, template, context,
                    text_template=None, text_body=None,
                    sender=None, recipients=None, cc=None, bcc=None,
                    inline_css=True,):
    """
    Sends an HTML e-mail and a text counterpart using Django's EmailMultiAlternatives
    class.
    @param subject: the subject line for the e-mail
    @type subject: unicode
    @param template: the django template that will be used to generate the
    HTML portion of the e-mail.
    @type template: basestring
    @param context: the context dictionary that will be used to render
    the HTML template
    @type context: dict[basestring, object]
    @param text_body: the text alternative for those e-mail clients that
    cannot view the HTML.
    @type text_body: basestring
    @param recipients: the recipients to whom the e-mail will be sent.
    If no recipients are specified, a default set of e-mails are used.
    @type: list[basestring]
    """
    html_body = loader.render_to_string(template, context)
    if inline_css:
        html_body = inlinecss.from_string(html_body, logger, )
    if text_body is None:
        if text_template is None:
            text_body = '<no text>'
        else:
            text_body = loader.render_to_string(text_template, context)
            # See https://code.djangoproject.com/ticket/2594
            text_body = os.linesep.join([s for s in text_body.splitlines() if s.strip()])
    sender = sender or DJENGA_FROM_EMAIL
    recipients = recipients or DJENGA_DEFAULT_RECIPIENTS
    try:
        msg = EmailMultiAlternatives(
            subject,
            text_body,
            sender,
            recipients,
            cc=cc,
            bcc=bcc
        )
        msg.attach_alternative(html_body, u'text/html')
        msg.send()
    except BaseException, ex: # pylint: disable=broad-except
        logger.exception(u'Exception encountered trying to send e-mail:  %s', ex)






from djenga.db.nested_transactions import commit_on_success
from djenga.db.nested_transactions import start_transaction
from djenga.db.nested_transactions import commit_transaction
from djenga.db.nested_transactions import rollback_transaction

__all__ = [
    'commit_on_success',
    'start_transaction',
    'commit_transaction',
    'rollback_transaction',
]




from __future__ import absolute_import
from djenga.management.commands.commandlogginglevels import VerbosityLevels
from djenga.management.commands.configurationmixin import ConfigurationMixin
from djenga.management.commands.csvmixin import CsvMixin
from djenga.management.commands.inputcsvmixin import InputCsvMixin
from djenga.management.commands.statuscommand import StatusCommand
from djenga.management.commands.run_integration_tests import Command as IntegrationTestCommand

__all__ = [
    'VerbosityLevels',
    'ConfigurationMixin',
    'CsvMixin',
    'InputCsvMixin',
    'StatusCommand',
    'IntegrationTestCommand',
]




from djenga.management.commands import StatusCommand
from djenga.models import ManagementCommand
from djenga.models import CommandOutput


class Command(StatusCommand):
    help = 'Cleans up the djenga.models.CommandOutput values by removing old entries'

    def __init__(self):
        super(Command, self).__init__()
        self.commands = []
        self.keep = None

    def add_arguments(self, parser):
        parser.add_argument(
            '-n', '--keep',
            help='Saves the last N runs of the command and delete the rest',
            type=int,
            default=10,
            dest='keep')

    def load_commands(self):
        self.commands = ManagementCommand.objects.all()

    def clean_command(self, command_id):
        rg_ids = list(CommandOutput.objects.filter(
            command_id=command_id
        ).order_by('-id').values_list('id', flat=True))
        if rg_ids and len(rg_ids) > self.keep:
            rg_ids = rg_ids[self.keep:]
            CommandOutput.objects.filter(
                id__in=rg_ids
            ).delete()

    def clean_commands(self):
        for x in self.commands:
            self.plain_log('Cleaning ')
            self.color_log(self.style.SQL_TABLE, x.name)
            self.plain_log('...')
            self.clean_command(x.id)
            self.color_log(self.style.SUCCESS, 'Done.\n')

    def handle(self, keep, *args, **options):
        self.keep = keep
        self.load_commands()
        self.clean_commands()
        return ''
